@@ -1,5 +1,5 @@
 /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-   Copyright (c) 2012-2019 The plumed team
+   Copyright (c) 2012-2023 The plumed team
    (see the PEOPLE file at the root of the distribution for a list of names)
 
    See http://www.plumed.org for more information.
@@ -35,6 +35,9 @@
 #include "tools/PDB.h"
 #include "tools/FileBase.h"
 #include "tools/IFile.h"
+#include "xdrfile/xdrfile_trr.h"
+#include "xdrfile/xdrfile_xtc.h"
+
 
 // when using molfile plugin
 #ifdef __PLUMED_HAS_MOLFILE_PLUGINS
@@ -52,13 +55,6 @@ using namespace PLMD::molfile;
 #endif
 #endif
 
-#ifdef __PLUMED_HAS_XDRFILE
-#include <xdrfile/xdrfile_trr.h>
-#include <xdrfile/xdrfile_xtc.h>
-#endif
-
-using namespace std;
-
 namespace PLMD {
 namespace cltools {
 
@@ -180,11 +176,11 @@ In addition, it allows \ref DUMPATOMS to write compressed xtc files.
 //
 
 #ifdef __PLUMED_HAS_MOLFILE_PLUGINS
-static vector<molfile_plugin_t *> plugins;
-static map <string, unsigned> pluginmap;
+static std::vector<molfile_plugin_t *> plugins;
+static std::map <std::string, unsigned> pluginmap;
 static int register_cb(void *v, vmdplugin_t *p) {
   //const char *key = p->name;
-  const auto ret = pluginmap.insert ( std::pair<string,unsigned>(string(p->name),plugins.size()) );
+  const auto ret = pluginmap.insert ( std::pair<std::string,unsigned>(std::string(p->name),plugins.size()) );
   if (ret.second==false) {
     //cerr<<"MOLFILE: found duplicate plugin for "<<key<<" : not inserted "<<endl;
   } else {
@@ -200,11 +196,11 @@ class Driver : public CLTool {
 public:
   static void registerKeywords( Keywords& keys );
   explicit Driver(const CLToolOptions& co );
-  int main(FILE* in,FILE*out,Communicator& pc);
+  int main(FILE* in,FILE*out,Communicator& pc) override;
   void evaluateNumericalDerivatives( const long int& step, PlumedMain& p, const std::vector<real>& coordinates,
                                      const std::vector<real>& masses, const std::vector<real>& charges,
                                      std::vector<real>& cell, const double& base, std::vector<real>& numder );
-  string description()const;
+  std::string description()const override;
 };
 
 template<typename real>
@@ -214,19 +210,18 @@ void Driver<real>::registerKeywords( Keywords& keys ) {
   keys.add("compulsory","--plumed","plumed.dat","specify the name of the plumed input file");
   keys.add("compulsory","--timestep","1.0","the timestep that was used in the calculation that produced this trajectory in picoseconds");
   keys.add("compulsory","--trajectory-stride","1","the frequency with which frames were output to this trajectory during the simulation"
-#ifdef __PLUMED_HAS_XDRFILE
            " (0 means that the number of the step is read from the trajectory file,"
            " currently working only for xtc/trr files read with --ixtc/--trr)"
-#endif
           );
   keys.add("compulsory","--multi","0","set number of replicas for multi environment (needs MPI)");
   keys.addFlag("--noatoms",false,"don't read in a trajectory.  Just use colvar files as specified in plumed.dat");
+  keys.addFlag("--parse-only",false,"read the plumed input file and stop");
+  keys.addFlag("--restart",false,"makes driver behave as if restarting");
   keys.add("atoms","--ixyz","the trajectory in xyz format");
   keys.add("atoms","--igro","the trajectory in gro format");
-#ifdef __PLUMED_HAS_XDRFILE
+  keys.add("atoms","--idlp4","the trajectory in DL_POLY_4 format");
   keys.add("atoms","--ixtc","the trajectory in xtc format (xdrfile implementation)");
   keys.add("atoms","--itrr","the trajectory in trr format (xdrfile implementation)");
-#endif
   keys.add("optional","--length-units","units for length, either as a string or a number");
   keys.add("optional","--mass-units","units for mass in pdb and mc file, either as a string or a number");
   keys.add("optional","--charge-units","units for charge in pdb and mc file, either as a string or a number");
@@ -250,9 +245,8 @@ void Driver<real>::registerKeywords( Keywords& keys ) {
   MOLFILE_INIT_ALL
   MOLFILE_REGISTER_ALL(NULL, register_cb)
   for(unsigned i=0; i<plugins.size(); i++) {
-    string kk="--mf_"+string(plugins[i]->name);
-    string mm=" molfile: the trajectory in "+string(plugins[i]->name)+" format " ;
-    //cerr<<"REGISTERING "<<kk<<mm<<endl;
+    std::string kk="--mf_"+std::string(plugins[i]->name);
+    std::string mm=" molfile: the trajectory in "+std::string(plugins[i]->name)+" format " ;
     keys.add("atoms",kk,mm);
   }
 #endif
@@ -264,7 +258,7 @@ Driver<real>::Driver(const CLToolOptions& co ):
   inputdata=commandline;
 }
 template<typename real>
-string Driver<real>::description()const { return "analyze trajectories with plumed"; }
+std::string Driver<real>::description()const { return "analyze trajectories with plumed"; }
 
 template<typename real>
 int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
@@ -275,16 +269,18 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
 // Parse everything
   bool printhelpdebug; parseFlag("--help-debug",printhelpdebug);
   if( printhelpdebug ) {
-    fprintf(out,"%s",
-            "Additional options for debug (only to be used in regtest):\n"
-            "  [--debug-float yes]     : turns on the single precision version (to check float interface)\n"
-            "  [--debug-dd yes]        : use a fake domain decomposition\n"
-            "  [--debug-pd yes]        : use a fake particle decomposition\n"
-           );
+    std::fprintf(out,"%s",
+                 "Additional options for debug (only to be used in regtest):\n"
+                 "  [--debug-float yes]     : turns on the single precision version (to check float interface)\n"
+                 "  [--debug-dd yes]        : use a fake domain decomposition\n"
+                 "  [--debug-pd yes]        : use a fake particle decomposition\n"
+                );
     return 0;
   }
   // Are we reading trajectory data
   bool noatoms; parseFlag("--noatoms",noatoms);
+  bool parseOnly; parseFlag("--parse-only",parseOnly);
+  bool restart; parseFlag("--restart",restart);
 
   std::string fakein;
   bool debug_float=false;
@@ -309,7 +305,7 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
     else if(fakein=="no") debug_pd=false;
     else error("--debug-pd should have argument yes or no");
   }
-  if(debug_pd) fprintf(out,"DEBUGGING PARTICLE DECOMPOSITION\n");
+  if(debug_pd) std::fprintf(out,"DEBUGGING PARTICLE DECOMPOSITION\n");
 
   bool debug_dd=false;
   fakein="";
@@ -318,7 +314,7 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
     else if(fakein=="no") debug_dd=false;
     else error("--debug-dd should have argument yes or no");
   }
-  if(debug_dd) fprintf(out,"DEBUGGING DOMAIN DECOMPOSITION\n");
+  if(debug_dd) std::fprintf(out,"DEBUGGING DOMAIN DECOMPOSITION\n");
 
   if( debug_pd || debug_dd ) {
     if(noatoms) error("cannot debug without atoms");
@@ -347,8 +343,8 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
     if(noatoms) error("must have atoms to debug_grex");
     if(multi<2)  error("--debug_grex needs --multi with at least two replicas");
     Tools::convert(fakein,grex_stride);
-    string n; Tools::convert(intercomm.Get_rank(),n);
-    string file;
+    std::string n; Tools::convert(intercomm.Get_rank(),n);
+    std::string file;
     parse("--debug-grex-log",file);
     if(file.length()>0) {
       file+="."+n;
@@ -357,14 +353,14 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
   }
 
 // Read the plumed input file name
-  string plumedFile; parse("--plumed",plumedFile);
+  std::string plumedFile; parse("--plumed",plumedFile);
 // the timestep
   double t; parse("--timestep",t);
   real timestep=real(t);
 // the stride
   unsigned stride; parse("--trajectory-stride",stride);
 // are we writing forces
-  string dumpforces(""), debugforces(""), dumpforcesFmt("%f");;
+  std::string dumpforces(""), debugforces(""), dumpforcesFmt("%f");;
   bool dumpfullvirial=false;
   if(!noatoms) {
     parse("--dump-forces",dumpforces);
@@ -377,7 +373,7 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
 
   real kt=-1.0;
   parse("--kt",kt);
-  string trajectory_fmt;
+  std::string trajectory_fmt;
 
   bool use_molfile=false;
 #ifdef __PLUMED_HAS_MOLFILE_PLUGINS
@@ -388,32 +384,32 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
 // it is necessary in order to store the pointer to ts_in.coords
   std::unique_ptr<float[]> ts_in_coords;
   ts_in.coords=ts_in_coords.get();
+  ts_in.velocities=NULL;
   ts_in.A=-1; // we use this to check whether cell is provided or not
 #endif
 
 // Read in an xyz file
-  string trajectoryFile(""), pdbfile(""), mcfile("");
-  bool pbc_cli_given=false; vector<double> pbc_cli_box(9,0.0);
+  std::string trajectoryFile(""), pdbfile(""), mcfile("");
+  bool pbc_cli_given=false; std::vector<double> pbc_cli_box(9,0.0);
   int command_line_natoms=-1;
 
   if(!noatoms) {
     std::string traj_xyz; parse("--ixyz",traj_xyz);
     std::string traj_gro; parse("--igro",traj_gro);
+    std::string traj_dlp4; parse("--idlp4",traj_dlp4);
     std::string traj_xtc;
     std::string traj_trr;
-#ifdef __PLUMED_HAS_XDRFILE
     parse("--ixtc",traj_xtc);
     parse("--itrr",traj_trr);
-#endif
 #ifdef __PLUMED_HAS_MOLFILE_PLUGINS
     for(unsigned i=0; i<plugins.size(); i++) {
-      string molfile_key="--mf_"+string(plugins[i]->name);
-      string traj_molfile;
+      std::string molfile_key="--mf_"+std::string(plugins[i]->name);
+      std::string traj_molfile;
       parse(molfile_key,traj_molfile);
       if(traj_molfile.length()>0) {
-        fprintf(out,"\nDRIVER: Found molfile format trajectory %s with name %s\n",plugins[i]->name,traj_molfile.c_str());
+        std::fprintf(out,"\nDRIVER: Found molfile format trajectory %s with name %s\n",plugins[i]->name,traj_molfile.c_str());
         trajectoryFile=traj_molfile;
-        trajectory_fmt=string(plugins[i]->name);
+        trajectory_fmt=std::string(plugins[i]->name);
         use_molfile=true;
         api = plugins[i];
       }
@@ -423,10 +419,11 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
       int nn=0;
       if(traj_xyz.length()>0) nn++;
       if(traj_gro.length()>0) nn++;
+      if(traj_dlp4.length()>0) nn++;
       if(traj_xtc.length()>0) nn++;
       if(traj_trr.length()>0) nn++;
       if(nn>1) {
-        fprintf(stderr,"ERROR: cannot provide more than one trajectory file\n");
+        std::fprintf(stderr,"ERROR: cannot provide more than one trajectory file\n");
         if(grex_log)fclose(grex_log);
         return 1;
       }
@@ -439,6 +436,10 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
       trajectoryFile=traj_gro;
       trajectory_fmt="gro";
     }
+    if(traj_dlp4.length()>0 && trajectoryFile.length()==0) {
+      trajectoryFile=traj_dlp4;
+      trajectory_fmt="dlp4";
+    }
     if(traj_xtc.length()>0 && trajectoryFile.length()==0) {
       trajectoryFile=traj_xtc;
       trajectory_fmt="xdr-xtc";
@@ -447,16 +448,16 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
       trajectoryFile=traj_trr;
       trajectory_fmt="xdr-trr";
     }
-    if(trajectoryFile.length()==0) {
-      fprintf(stderr,"ERROR: missing trajectory data\n");
+    if(trajectoryFile.length()==0&&!parseOnly) {
+      std::fprintf(stderr,"ERROR: missing trajectory data\n");
       if(grex_log)fclose(grex_log);
       return 1;
     }
-    string lengthUnits(""); parse("--length-units",lengthUnits);
+    std::string lengthUnits(""); parse("--length-units",lengthUnits);
     if(lengthUnits.length()>0) units.setLength(lengthUnits);
-    string chargeUnits(""); parse("--charge-units",chargeUnits);
+    std::string chargeUnits(""); parse("--charge-units",chargeUnits);
     if(chargeUnits.length()>0) units.setCharge(chargeUnits);
-    string massUnits(""); parse("--mass-units",massUnits);
+    std::string massUnits(""); parse("--mass-units",massUnits);
     if(massUnits.length()>0) units.setMass(massUnits);
 
     parse("--pdb",pdbfile);
@@ -467,17 +468,17 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
 
     parse("--mc",mcfile);
 
-    string pbc_cli_list; parse("--box",pbc_cli_list);
+    std::string pbc_cli_list; parse("--box",pbc_cli_list);
     if(pbc_cli_list.length()>0) {
       pbc_cli_given=true;
-      vector<string> words=Tools::getWords(pbc_cli_list,",");
+      std::vector<std::string> words=Tools::getWords(pbc_cli_list,",");
       if(words.size()==3) {
-        for(int i=0; i<3; i++) sscanf(words[i].c_str(),"%100lf",&(pbc_cli_box[4*i]));
+        for(int i=0; i<3; i++) std::sscanf(words[i].c_str(),"%100lf",&(pbc_cli_box[4*i]));
       } else if(words.size()==9) {
-        for(int i=0; i<9; i++) sscanf(words[i].c_str(),"%100lf",&(pbc_cli_box[i]));
+        for(int i=0; i<9; i++) std::sscanf(words[i].c_str(),"%100lf",&(pbc_cli_box[i]));
       } else {
-        string msg="ERROR: cannot parse command-line box "+pbc_cli_list;
-        fprintf(stderr,"%s\n",msg.c_str());
+        std::string msg="ERROR: cannot parse command-line box "+pbc_cli_list;
+        std::fprintf(stderr,"%s\n",msg.c_str());
         return 1;
       }
 
@@ -494,12 +495,13 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
   }
 
   PlumedMain p;
-  int rr=sizeof(real);
-  p.cmd("setRealPrecision",&rr);
+  p.cmd("setRealPrecision",(int)sizeof(real));
   int checknatoms=-1;
   long int step=0;
   parse("--initial-step",step);
 
+  if(restart) p.cmd("setRestart",1);
+
   if(Communicator::initialized()) {
     if(multi) {
       if(intracomm.Get_rank()==0) p.cmd("GREX setMPIIntercomm",&intercomm.Get_comm());
@@ -508,26 +510,32 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
     }
     p.cmd("setMPIComm",&intracomm.Get_comm());
   }
-  p.cmd("setMDLengthUnits",&units.getLength());
-  p.cmd("setMDChargeUnits",&units.getCharge());
-  p.cmd("setMDMassUnits",&units.getMass());
+  p.cmd("setMDLengthUnits",units.getLength());
+  p.cmd("setMDChargeUnits",units.getCharge());
+  p.cmd("setMDMassUnits",units.getMass());
   p.cmd("setMDEngine","driver");
-  p.cmd("setTimestep",&timestep);
+  p.cmd("setTimestep",timestep);
   p.cmd("setPlumedDat",plumedFile.c_str());
   p.cmd("setLog",out);
 
   int natoms;
+  int lvl=0;
+  int pb=1;
+
+  if(parseOnly) {
+    if(command_line_natoms<0) error("--parseOnly requires setting the number of atoms with --natoms");
+    natoms=command_line_natoms;
+  }
+
 
   FILE* fp=NULL; FILE* fp_forces=NULL; OFile fp_dforces;
-#ifdef __PLUMED_HAS_XDRFILE
-  XDRFILE* xd=NULL;
-#endif
-  if(!noatoms) {
+  xdrfile::XDRFILE* xd=NULL;
+  if(!noatoms&&!parseOnly) {
     if (trajectoryFile=="-")
       fp=in;
     else {
       if(multi) {
-        string n;
+        std::string n;
         Tools::convert(intercomm.Get_rank(),n);
         std::string testfile=FileBase::appendSuffix(trajectoryFile,"."+n);
         FILE* tmp_fp=fopen(testfile.c_str(),"r");
@@ -540,32 +548,30 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
           if(command_line_natoms>=0) natoms=command_line_natoms;
           else error("this file format does not provide number of atoms; use --natoms on the command line");
         }
-        ts_in_coords.reset(new float [3*natoms]);
+        ts_in_coords=Tools::make_unique<float[]>(3*natoms);
         ts_in.coords = ts_in_coords.get();
 #endif
       } else if(trajectory_fmt=="xdr-xtc" || trajectory_fmt=="xdr-trr") {
-#ifdef __PLUMED_HAS_XDRFILE
-        xd=xdrfile_open(trajectoryFile.c_str(),"r");
+        xd=xdrfile::xdrfile_open(trajectoryFile.c_str(),"r");
         if(!xd) {
-          string msg="ERROR: Error opening trajectory file "+trajectoryFile;
-          fprintf(stderr,"%s\n",msg.c_str());
+          std::string msg="ERROR: Error opening trajectory file "+trajectoryFile;
+          std::fprintf(stderr,"%s\n",msg.c_str());
           return 1;
         }
-        if(trajectory_fmt=="xdr-xtc") read_xtc_natoms(&trajectoryFile[0],&natoms);
-        if(trajectory_fmt=="xdr-trr") read_trr_natoms(&trajectoryFile[0],&natoms);
-#endif
+        if(trajectory_fmt=="xdr-xtc") xdrfile::read_xtc_natoms(&trajectoryFile[0],&natoms);
+        if(trajectory_fmt=="xdr-trr") xdrfile::read_trr_natoms(&trajectoryFile[0],&natoms);
       } else {
         fp=fopen(trajectoryFile.c_str(),"r");
         if(!fp) {
-          string msg="ERROR: Error opening trajectory file "+trajectoryFile;
-          fprintf(stderr,"%s\n",msg.c_str());
+          std::string msg="ERROR: Error opening trajectory file "+trajectoryFile;
+          std::fprintf(stderr,"%s\n",msg.c_str());
           return 1;
         }
       }
     }
     if(dumpforces.length()>0) {
       if(Communicator::initialized() && pc.Get_size()>1) {
-        string n;
+        std::string n;
         Tools::convert(pc.Get_rank(),n);
         dumpforces+="."+n;
       }
@@ -573,7 +579,7 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
     }
     if(debugforces.length()>0) {
       if(Communicator::initialized() && pc.Get_size()>1) {
-        string n;
+        std::string n;
         Tools::convert(pc.Get_rank(),n);
         debugforces+="."+n;
       }
@@ -591,8 +597,8 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
   std::vector<real> numder;
 
 // variables to test particle decomposition
-  int pd_nlocal;
-  int pd_start;
+  int pd_nlocal=0;
+  int pd_start=0;
 // variables to test random decomposition (=domain decomposition)
   std::vector<int>  dd_gatindex;
   std::vector<int>  dd_g2l;
@@ -600,34 +606,46 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
   std::vector<real> dd_charges;
   std::vector<real> dd_forces;
   std::vector<real> dd_coordinates;
-  int dd_nlocal;
+  int dd_nlocal=0;
 // random stream to choose decompositions
   Random rnd;
 
+  if(trajectory_fmt=="dlp4") {
+    if(!Tools::getline(fp,line)) error("error reading title");
+    if(!Tools::getline(fp,line)) error("error reading atoms");
+    std::sscanf(line.c_str(),"%d %d %d",&lvl,&pb,&natoms);
+
+  }
+  bool lstep=true;
   while(true) {
-    if(!noatoms) {
+    if(!noatoms&&!parseOnly) {
       if(use_molfile==true) {
 #ifdef __PLUMED_HAS_MOLFILE_PLUGINS
         int rc;
         rc = api->read_next_timestep(h_in, natoms, &ts_in);
-        //if(rc==MOLFILE_SUCCESS){
-        //       printf(" read this one :success \n");
-        //}
         if(rc==MOLFILE_EOF) {
-          //printf(" read this one :eof or error \n");
           break;
         }
 #endif
-      } else if(trajectory_fmt=="xyz" || trajectory_fmt=="gro") {
+      } else if(trajectory_fmt=="xyz" || trajectory_fmt=="gro" || trajectory_fmt=="dlp4") {
         if(!Tools::getline(fp,line)) break;
       }
     }
-
     bool first_step=false;
-    if(!noatoms) {
+    if(!noatoms&&!parseOnly) {
       if(use_molfile==false && (trajectory_fmt=="xyz" || trajectory_fmt=="gro")) {
         if(trajectory_fmt=="gro") if(!Tools::getline(fp,line)) error("premature end of trajectory file");
-        sscanf(line.c_str(),"%100d",&natoms);
+        std::sscanf(line.c_str(),"%100d",&natoms);
+      }
+      if(use_molfile==false && trajectory_fmt=="dlp4") {
+        char xa[9];
+        int xb,xc,xd;
+        double t;
+        std::sscanf(line.c_str(),"%8s %ld %d %d %d %lf",xa,&step,&xb,&xc,&xd,&t);
+        if (lstep) {
+          p.cmd("setTimestep",real(t));
+          lstep = false;
+        }
       }
     }
     if(checknatoms<0 && !noatoms) {
@@ -660,11 +678,12 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
     }
     if( checknatoms<0 ) {
       if(kt>=0) {
-        p.cmd("setKbT",&kt);
+        p.cmd("setKbT",kt);
       }
       checknatoms=natoms;
-      p.cmd("setNatoms",&natoms);
+      p.cmd("setNatoms",natoms);
       p.cmd("init");
+      if(parseOnly) break;
     }
     if(checknatoms!=natoms) {
       std::string stepstr; Tools::convert(step,stepstr);
@@ -679,8 +698,8 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
     if( first_step || rnd.U01()>0.5) {
       if(debug_pd) {
         int npe=intracomm.Get_size();
-        vector<int> loc(npe,0);
-        vector<int> start(npe,0);
+        std::vector<int> loc(npe,0);
+        std::vector<int> start(npe,0);
         for(int i=0; i<npe-1; i++) {
           int cc=(natoms*2*rnd.U01())/npe;
           if(start[i]+cc>natoms) cc=natoms-start[i];
@@ -693,12 +712,12 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
         pd_nlocal=loc[intracomm.Get_rank()];
         pd_start=start[intracomm.Get_rank()];
         if(intracomm.Get_rank()==0) {
-          fprintf(out,"\nDRIVER: Reassigning particle decomposition\n");
-          fprintf(out,"DRIVER: "); for(int i=0; i<npe; i++) fprintf(out,"%d ",loc[i]); printf("\n");
-          fprintf(out,"DRIVER: "); for(int i=0; i<npe; i++) fprintf(out,"%d ",start[i]); printf("\n");
+          std::fprintf(out,"\nDRIVER: Reassigning particle decomposition\n");
+          std::fprintf(out,"DRIVER: "); for(int i=0; i<npe; i++) std::fprintf(out,"%d ",loc[i]); printf("\n");
+          std::fprintf(out,"DRIVER: "); for(int i=0; i<npe; i++) std::fprintf(out,"%d ",start[i]); printf("\n");
         }
-        p.cmd("setAtomsNlocal",&pd_nlocal);
-        p.cmd("setAtomsContiguous",&pd_start);
+        p.cmd("setAtomsNlocal",pd_nlocal);
+        p.cmd("setAtomsContiguous",pd_start);
       } else if(debug_dd) {
         int npe=intracomm.Get_size();
         int rank=intracomm.Get_rank();
@@ -722,10 +741,10 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
           }
         }
         if(intracomm.Get_rank()==0) {
-          fprintf(out,"\nDRIVER: Reassigning domain decomposition\n");
+          std::fprintf(out,"\nDRIVER: Reassigning domain decomposition\n");
         }
-        p.cmd("setAtomsNlocal",&dd_nlocal);
-        p.cmd("setAtomsGatindex",&dd_gatindex[0]);
+        p.cmd("setAtomsNlocal",dd_nlocal);
+        p.cmd("setAtomsGatindex",&dd_gatindex[0],dd_nlocal);
       }
     }
 
@@ -736,17 +755,17 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
         if(pbc_cli_given==false) {
           if(ts_in.A>0.0) { // this is negative if molfile does not provide box
             // info on the cell: convert using pbcset.tcl from pbctools in vmd distribution
-            real cosBC=cos(ts_in.alpha*pi/180.);
-            //double sinBC=sin(ts_in.alpha*pi/180.);
-            real cosAC=cos(ts_in.beta*pi/180.);
-            real cosAB=cos(ts_in.gamma*pi/180.);
-            real sinAB=sin(ts_in.gamma*pi/180.);
-            real Ax=ts_in.A;
-            real Bx=ts_in.B*cosAB;
-            real By=ts_in.B*sinAB;
-            real Cx=ts_in.C*cosAC;
-            real Cy=(ts_in.C*ts_in.B*cosBC-Cx*Bx)/By;
-            real Cz=sqrt(ts_in.C*ts_in.C-Cx*Cx-Cy*Cy);
+            real cosBC=cos(real(ts_in.alpha)*pi/180.);
+            //double sinBC=std::sin(ts_in.alpha*pi/180.);
+            real cosAC=std::cos(real(ts_in.beta)*pi/180.);
+            real cosAB=std::cos(real(ts_in.gamma)*pi/180.);
+            real sinAB=std::sin(real(ts_in.gamma)*pi/180.);
+            real Ax=real(ts_in.A);
+            real Bx=real(ts_in.B)*cosAB;
+            real By=real(ts_in.B)*sinAB;
+            real Cx=real(ts_in.C)*cosAC;
+            real Cy=(real(ts_in.C)*real(ts_in.B)*cosBC-Cx*Bx)/By;
+            real Cz=std::sqrt(real(ts_in.C)*real(ts_in.C)-Cx*Cx-Cy*Cy);
             cell[0]=Ax/10.; cell[1]=0.; cell[2]=0.;
             cell[3]=Bx/10.; cell[4]=By/10.; cell[5]=0.;
             cell[6]=Cx/10.; cell[7]=Cy/10.; cell[8]=Cz/10.;
@@ -761,27 +780,25 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
         // info on coords
         // the order is xyzxyz...
         for(int i=0; i<3*natoms; i++) {
-          coordinates[i]=real(ts_in.coords[i]/10.); //convert to nm
+          coordinates[i]=real(ts_in.coords[i])/real(10.); //convert to nm
           //cerr<<"COOR "<<coordinates[i]<<endl;
         }
 #endif
       } else if(trajectory_fmt=="xdr-xtc" || trajectory_fmt=="xdr-trr") {
-#ifdef __PLUMED_HAS_XDRFILE
         int localstep;
         float time;
-        matrix box;
-        std::unique_ptr<rvec[]> pos(new rvec[natoms]);
+        xdrfile::matrix box;
+        auto pos=Tools::make_unique<xdrfile::rvec[]>(natoms);
         float prec,lambda;
-        int ret=exdrOK;
-        if(trajectory_fmt=="xdr-xtc") ret=read_xtc(xd,natoms,&localstep,&time,box,pos.get(),&prec);
-        if(trajectory_fmt=="xdr-trr") ret=read_trr(xd,natoms,&localstep,&time,&lambda,box,pos.get(),NULL,NULL);
+        int ret=xdrfile::exdrOK;
+        if(trajectory_fmt=="xdr-xtc") ret=xdrfile::read_xtc(xd,natoms,&localstep,&time,box,pos.get(),&prec);
+        if(trajectory_fmt=="xdr-trr") ret=xdrfile::read_trr(xd,natoms,&localstep,&time,&lambda,box,pos.get(),NULL,NULL);
         if(stride==0) step=localstep;
-        if(ret==exdrENDOFFILE) break;
-        if(ret!=exdrOK) break;
+        if(ret==xdrfile::exdrENDOFFILE) break;
+        if(ret!=xdrfile::exdrOK) break;
         for(unsigned i=0; i<3; i++) for(unsigned j=0; j<3; j++) cell[3*i+j]=box[i][j];
         for(int i=0; i<natoms; i++) for(unsigned j=0; j<3; j++)
             coordinates[3*i+j]=real(pos[i][j]);
-#endif
       } else {
         if(trajectory_fmt=="xyz") {
           if(!Tools::getline(fp,line)) error("premature end of trajectory file");
@@ -791,18 +808,32 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
             std::vector<std::string> words;
             words=Tools::getWords(line);
             if(words.size()==3) {
-              sscanf(line.c_str(),"%100lf %100lf %100lf",&celld[0],&celld[4],&celld[8]);
+              std::sscanf(line.c_str(),"%100lf %100lf %100lf",&celld[0],&celld[4],&celld[8]);
             } else if(words.size()==9) {
-              sscanf(line.c_str(),"%100lf %100lf %100lf %100lf %100lf %100lf %100lf %100lf %100lf",
-                     &celld[0], &celld[1], &celld[2],
-                     &celld[3], &celld[4], &celld[5],
-                     &celld[6], &celld[7], &celld[8]);
+              std::sscanf(line.c_str(),"%100lf %100lf %100lf %100lf %100lf %100lf %100lf %100lf %100lf",
+                          &celld[0], &celld[1], &celld[2],
+                          &celld[3], &celld[4], &celld[5],
+                          &celld[6], &celld[7], &celld[8]);
             } else error("needed box in second line of xyz file");
           } else {			// from command line
             celld=pbc_cli_box;
           }
           for(unsigned i=0; i<9; i++)cell[i]=real(celld[i]);
         }
+        if(trajectory_fmt=="dlp4") {
+          std::vector<double> celld(9,0.0);
+          if(pbc_cli_given==false) {
+            if(!Tools::getline(fp,line)) error("error reading vector a of cell");
+            std::sscanf(line.c_str(),"%lf %lf %lf",&celld[0],&celld[1],&celld[2]);
+            if(!Tools::getline(fp,line)) error("error reading vector b of cell");
+            std::sscanf(line.c_str(),"%lf %lf %lf",&celld[3],&celld[4],&celld[5]);
+            if(!Tools::getline(fp,line)) error("error reading vector c of cell");
+            std::sscanf(line.c_str(),"%lf %lf %lf",&celld[6],&celld[7],&celld[8]);
+          } else {
+            celld=pbc_cli_box;
+          }
+          for(auto i=0; i<9; i++)cell[i]=real(celld[i])*0.1;
+        }
         int ddist=0;
         // Read coordinates
         for(int i=0; i<natoms; i++) {
@@ -820,18 +851,36 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
               // calculate the distance between dots (as in gromacs gmxlib/confio.c, routine get_w_conf )
               //
               const char      *p1, *p2, *p3;
-              p1 = strchr(line.c_str(), '.');
+              p1 = std::strchr(line.c_str(), '.');
               if (p1 == NULL) error("seems there are no coordinates in the gro file");
-              p2 = strchr(&p1[1], '.');
+              p2 = std::strchr(&p1[1], '.');
               if (p2 == NULL) error("seems there is only one coordinates in the gro file");
               ddist = p2 - p1;
-              p3 = strchr(&p2[1], '.');
+              p3 = std::strchr(&p2[1], '.');
               if (p3 == NULL)error("seems there are only two coordinates in the gro file");
               if (p3 - p2 != ddist)error("not uniform spacing in fields in the gro file");
             }
             Tools::convert(line.substr(20,ddist),cc[0]);
             Tools::convert(line.substr(20+ddist,ddist),cc[1]);
             Tools::convert(line.substr(20+ddist+ddist,ddist),cc[2]);
+          } else if(trajectory_fmt=="dlp4") {
+            char dummy[9];
+            int idummy;
+            double m,c;
+            std::sscanf(line.c_str(),"%8s %d %lf %lf",dummy,&idummy,&m,&c);
+            masses[i]=real(m);
+            charges[i]=real(c);
+            if(!Tools::getline(fp,line)) error("error reading coordinates");
+            std::sscanf(line.c_str(),"%lf %lf %lf",&cc[0],&cc[1],&cc[2]);
+            cc[0]*=0.1;
+            cc[1]*=0.1;
+            cc[2]*=0.1;
+            if(lvl>0) {
+              if(!Tools::getline(fp,line)) error("error skipping velocities");
+            }
+            if(lvl>1) {
+              if(!Tools::getline(fp,line)) error("error skipping forces");
+            }
           } else plumed_error();
           if(!debug_pd || ( i>=pd_start && i<pd_start+pd_nlocal) ) {
             coordinates[3*i]=real(cc[0]);
@@ -841,7 +890,7 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
         }
         if(trajectory_fmt=="gro") {
           if(!Tools::getline(fp,line)) error("premature end of trajectory file");
-          std::vector<string> words=Tools::getWords(line);
+          std::vector<std::string> words=Tools::getWords(line);
           if(words.size()<3) error("cannot understand box format");
           Tools::convert(words[0],cell[0]);
           Tools::convert(words[1],cell[4]);
@@ -856,7 +905,7 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
 
       }
 
-      p.cmd("setStepLong",&step);
+      p.cmd("setStepLong",step);
       p.cmd("setStopFlag",&plumedStopCondition);
 
       if(debug_dd) {
@@ -866,27 +915,33 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
           dd_coordinates[3*i+1]=coordinates[3*kk+1];
           dd_coordinates[3*i+2]=coordinates[3*kk+2];
         }
-        p.cmd("setForces",&dd_forces[0]);
-        p.cmd("setPositions",&dd_coordinates[0]);
-        p.cmd("setMasses",&dd_masses[0]);
-        p.cmd("setCharges",&dd_charges[0]);
+        p.cmd("setForces",&dd_forces[0],3*dd_nlocal);
+        p.cmd("setPositions",&dd_coordinates[0],3*dd_nlocal);
+        p.cmd("setMasses",&dd_masses[0],dd_nlocal);
+        p.cmd("setCharges",&dd_charges[0],dd_nlocal);
       } else {
 // this is required to avoid troubles when the last domain
 // contains zero atoms
 // Basically, for empty domains we pass null pointers
 #define fix_pd(xx) (pd_nlocal!=0?&xx:NULL)
-        p.cmd("setForces",fix_pd(forces[3*pd_start]));
-        p.cmd("setPositions",fix_pd(coordinates[3*pd_start]));
-        p.cmd("setMasses",fix_pd(masses[pd_start]));
-        p.cmd("setCharges",fix_pd(charges[pd_start]));
+        p.cmd("setForces",fix_pd(forces[3*pd_start]),3*pd_nlocal);
+        p.cmd("setPositions",fix_pd(coordinates[3*pd_start]),3*pd_nlocal);
+        p.cmd("setMasses",fix_pd(masses[pd_start]),pd_nlocal);
+        p.cmd("setCharges",fix_pd(charges[pd_start]),pd_nlocal);
       }
-      p.cmd("setBox",&cell[0]);
-      p.cmd("setVirial",&virial[0]);
+      p.cmd("setBox",cell.data(),9);
+      p.cmd("setVirial",virial.data(),9);
     } else {
-      p.cmd("setStepLong",&step);
+      p.cmd("setStepLong",step);
       p.cmd("setStopFlag",&plumedStopCondition);
     }
     p.cmd("calc");
+    if(debugforces.length()>0) {
+      virial.assign(9,real(0.0));
+      forces.assign(3*natoms,real(0.0));
+      p.cmd("prepareCalc");
+      p.cmd("performCalcNoUpdate");
+    }
 
 // this is necessary as only processor zero is adding to the virial:
     intracomm.Bcast(virial,0);
@@ -910,37 +965,37 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
         int partner=r+(2*((r+step/grex_stride)%2))-1;
         if(partner<0)partner=0;
         if(partner>=n) partner=n-1;
-        p.cmd("GREX setPartner",&partner);
+        p.cmd("GREX setPartner",partner);
         p.cmd("GREX calculate");
         p.cmd("GREX shareAllDeltaBias");
         for(int i=0; i<n; i++) {
-          string s; Tools::convert(i,s);
-          real a=std::numeric_limits<real>::quiet_NaN(); s="GREX getDeltaBias "+s; p.cmd(s.c_str(),&a);
-          if(grex_log) fprintf(grex_log," %f",a);
+          std::string s; Tools::convert(i,s);
+          real a=std::numeric_limits<real>::quiet_NaN(); s="GREX getDeltaBias "+s; p.cmd(s,&a);
+          if(grex_log) std::fprintf(grex_log," %f",a);
         }
-        if(grex_log) fprintf(grex_log,"\n");
+        if(grex_log) std::fprintf(grex_log,"\n");
       }
     }
 
 
     if(fp_forces) {
-      fprintf(fp_forces,"%d\n",natoms);
-      string fmtv=dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+"\n";
-      string fmt=dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+"\n";
+      std::fprintf(fp_forces,"%d\n",natoms);
+      std::string fmtv=dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+"\n";
+      std::string fmt=dumpforcesFmt+" "+dumpforcesFmt+" "+dumpforcesFmt+"\n";
       if(dumpfullvirial) {
-        fprintf(fp_forces,fmtv.c_str(),virial[0],virial[1],virial[2],virial[3],virial[4],virial[5],virial[6],virial[7],virial[8]);
+        std::fprintf(fp_forces,fmtv.c_str(),virial[0],virial[1],virial[2],virial[3],virial[4],virial[5],virial[6],virial[7],virial[8]);
       } else {
-        fprintf(fp_forces,fmt.c_str(),virial[0],virial[4],virial[8]);
+        std::fprintf(fp_forces,fmt.c_str(),virial[0],virial[4],virial[8]);
       }
       fmt="X "+fmt;
       for(int i=0; i<natoms; i++)
-        fprintf(fp_forces,fmt.c_str(),forces[3*i],forces[3*i+1],forces[3*i+2]);
+        std::fprintf(fp_forces,fmt.c_str(),forces[3*i],forces[3*i+1],forces[3*i+2]);
     }
     if(debugforces.length()>0) {
       // Now call the routine to work out the derivatives numerically
       numder.assign(3*natoms+9,real(0.0)); real base=0;
       p.cmd("getBias",&base);
-      if( fabs(base)<epsilon ) printf("WARNING: bias for configuration appears to be zero so debugging forces is trivial");
+      if( std::abs(base)<epsilon ) printf("WARNING: bias for configuration appears to be zero so debugging forces is trivial");
       evaluateNumericalDerivatives( step, p, coordinates, masses, charges, cell, base, numder );
 
       // And output everything to a file
@@ -964,14 +1019,12 @@ int Driver<real>::main(FILE* in,FILE*out,Communicator& pc) {
 
     step+=stride;
   }
-  p.cmd("runFinalJobs");
+  if(!parseOnly) p.cmd("runFinalJobs");
 
   if(fp_forces) fclose(fp_forces);
   if(debugforces.length()>0) fp_dforces.close();
   if(fp && fp!=in)fclose(fp);
-#ifdef __PLUMED_HAS_XDRFILE
-  if(xd) xdrfile_close(xd);
-#endif
+  if(xd) xdrfile::xdrfile_close(xd);
 #ifdef __PLUMED_HAS_MOLFILE_PLUGINS
   if(h_in) api->close_file_read(h_in);
 #endif
@@ -985,7 +1038,7 @@ void Driver<real>::evaluateNumericalDerivatives( const long int& step, PlumedMai
     const std::vector<real>& masses, const std::vector<real>& charges,
     std::vector<real>& cell, const double& base, std::vector<real>& numder ) {
 
-  int natoms = coordinates.size() / 3; real delta = sqrt(epsilon);
+  int natoms = coordinates.size() / 3; real delta = std::sqrt(epsilon);
   std::vector<Vector> pos(natoms); real bias=0;
   std::vector<real> fake_forces( 3*natoms ), fake_virial(9);
   for(int i=0; i<natoms; ++i) {
@@ -995,13 +1048,13 @@ void Driver<real>::evaluateNumericalDerivatives( const long int& step, PlumedMai
   for(int i=0; i<natoms; ++i) {
     for(unsigned j=0; j<3; ++j) {
       pos[i][j]=pos[i][j]+delta;
-      p.cmd("setStepLong",&step);
-      p.cmd("setPositions",&pos[0][0]);
-      p.cmd("setForces",&fake_forces[0]);
-      p.cmd("setMasses",&masses[0]);
-      p.cmd("setCharges",&charges[0]);
-      p.cmd("setBox",&cell[0]);
-      p.cmd("setVirial",&fake_virial[0]);
+      p.cmd("setStepLong",step);
+      p.cmd("setPositions",&pos[0][0],3*natoms);
+      p.cmd("setForces",&fake_forces[0],3*natoms);
+      p.cmd("setMasses",&masses[0],natoms);
+      p.cmd("setCharges",&charges[0],natoms);
+      p.cmd("setBox",&cell[0],9);
+      p.cmd("setVirial",&fake_virial[0],9);
       p.cmd("prepareCalc");
       p.cmd("performCalcNoUpdate");
       p.cmd("getBias",&bias);
@@ -1019,13 +1072,13 @@ void Driver<real>::evaluateNumericalDerivatives( const long int& step, PlumedMai
       for(int j=0; j<natoms; ++j) pos[j]=pbc.realToScaled( pos[j] );
       cell[3*i+k]=box(i,k)=box(i,k)+delta; pbc.setBox(box);
       for(int j=0; j<natoms; j++) pos[j]=pbc.scaledToReal( pos[j] );
-      p.cmd("setStepLong",&step);
-      p.cmd("setPositions",&pos[0][0]);
-      p.cmd("setForces",&fake_forces[0]);
-      p.cmd("setMasses",&masses[0]);
-      p.cmd("setCharges",&charges[0]);
-      p.cmd("setBox",&cell[0]);
-      p.cmd("setVirial",&fake_virial[0]);
+      p.cmd("setStepLong",step);
+      p.cmd("setPositions",&pos[0][0],3*natoms);
+      p.cmd("setForces",&fake_forces[0],3*natoms);
+      p.cmd("setMasses",&masses[0],natoms);
+      p.cmd("setCharges",&charges[0],natoms);
+      p.cmd("setBox",&cell[0],9);
+      p.cmd("setVirial",&fake_virial[0],9);
       p.cmd("prepareCalc");
       p.cmd("performCalcNoUpdate");
       p.cmd("getBias",&bias);
