@@ -1,5 +1,5 @@
 /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-   Copyright (c) 2011-2019 The plumed team
+   Copyright (c) 2011-2023 The plumed team
    (see the PEOPLE file at the root of the distribution for a list of names)
 
    See http://www.plumed.org for more information.
@@ -37,13 +37,12 @@
 #include <cfloat>
 #include <array>
 
-using namespace std;
 namespace PLMD {
 
-constexpr size_t Grid::maxdim;
+constexpr std::size_t GridBase::maxdim;
 
-Grid::Grid(const std::string& funcl, const std::vector<Value*> & args, const vector<std::string> & gmin,
-           const vector<std::string> & gmax, const vector<unsigned> & nbin, bool dospline, bool usederiv, bool doclear) {
+GridBase::GridBase(const std::string& funcl, const std::vector<Value*> & args, const std::vector<std::string> & gmin,
+                   const std::vector<std::string> & gmax, const std::vector<unsigned> & nbin, bool dospline, bool usederiv) {
 // various checks
   plumed_assert(args.size()<=maxdim) << "grid dim cannot exceed "<<maxdim;
   plumed_massert(args.size()==gmin.size(),"grid min dimensions in input do not match number of arguments");
@@ -52,7 +51,7 @@ Grid::Grid(const std::string& funcl, const std::vector<Value*> & args, const vec
   unsigned dim=gmax.size();
   std::vector<std::string> names;
   std::vector<bool> isperiodic;
-  std::vector<string> pmin,pmax;
+  std::vector<std::string> pmin,pmax;
   names.resize( dim );
   isperiodic.resize( dim );
   pmin.resize( dim );
@@ -69,19 +68,20 @@ Grid::Grid(const std::string& funcl, const std::vector<Value*> & args, const vec
     }
   }
 // this is a value-independent initializator
-  Init(funcl,names,gmin,gmax,nbin,dospline,usederiv,doclear,isperiodic,pmin,pmax);
+  Init(funcl,names,gmin,gmax,nbin,dospline,usederiv,isperiodic,pmin,pmax);
 }
 
-Grid::Grid(const std::string& funcl, const std::vector<string> &names, const std::vector<std::string> & gmin,
-           const vector<std::string> & gmax, const std::vector<unsigned> & nbin, bool dospline, bool usederiv, bool doclear, const std::vector<bool> &isperiodic, const std::vector<std::string> &pmin, const std::vector<std::string> &pmax ) {
+GridBase::GridBase(const std::string& funcl, const std::vector<std::string> &names, const std::vector<std::string> & gmin,
+                   const std::vector<std::string> & gmax, const std::vector<unsigned> & nbin, bool dospline, bool usederiv,
+                   const std::vector<bool> &isperiodic, const std::vector<std::string> &pmin, const std::vector<std::string> &pmax ) {
 // this calls the initializator
-  Init(funcl,names,gmin,gmax,nbin,dospline,usederiv,doclear,isperiodic,pmin,pmax);
+  Init(funcl,names,gmin,gmax,nbin,dospline,usederiv,isperiodic,pmin,pmax);
 }
 
-void Grid::Init(const std::string& funcl, const std::vector<std::string> &names, const vector<std::string> & gmin,
-                const std::vector<std::string> & gmax, const std::vector<unsigned> & nbin, bool dospline, bool usederiv, bool doclear,
-                const std::vector<bool> &isperiodic, const std::vector<std::string> &pmin, const std::vector<std::string> &pmax ) {
-  contour_location=0.0; fmt_="%14.9f";
+void GridBase::Init(const std::string& funcl, const std::vector<std::string> &names, const std::vector<std::string> & gmin,
+                    const std::vector<std::string> & gmax, const std::vector<unsigned> & nbin, bool dospline, bool usederiv,
+                    const std::vector<bool> &isperiodic, const std::vector<std::string> &pmin, const std::vector<std::string> &pmax ) {
+  fmt_="%14.9f";
 // various checks
   plumed_assert(names.size()<=maxdim) << "grid size cannot exceed "<<maxdim;
   plumed_massert(names.size()==gmin.size(),"grid dimensions in input do not match number of arguments");
@@ -118,66 +118,48 @@ void Grid::Init(const std::string& funcl, const std::vector<std::string> &names,
     if( !pbc_[i] ) { max_[i] += dx_[i]; nbin_[i] += 1; }
     maxsize_*=nbin_[i];
   }
-  if(doclear) clear();
 }
 
-void Grid::clear() {
-  grid_.resize(maxsize_);
-  if(usederiv_) der_.resize(maxsize_);
-  for(index_t i=0; i<maxsize_; ++i) {
-    grid_[i]=0.0;
-    if(usederiv_) {
-      (der_[i]).resize(dimension_);
-      for(unsigned int j=0; j<dimension_; ++j) der_[i][j]=0.0;
-    }
-  }
-}
-
-vector<std::string> Grid::getMin() const {
+std::vector<std::string> GridBase::getMin() const {
   return str_min_;
 }
 
-vector<std::string> Grid::getMax() const {
+std::vector<std::string> GridBase::getMax() const {
   return str_max_;
 }
 
-vector<double> Grid::getDx() const {
+std::vector<double> GridBase::getDx() const {
   return dx_;
 }
 
-double Grid::getDx(index_t j) const {
+double GridBase::getDx(index_t j) const {
   return dx_[j];
 }
 
-double Grid::getBinVolume() const {
+double GridBase::getBinVolume() const {
   double vol=1.;
   for(unsigned i=0; i<dx_.size(); ++i) vol*=dx_[i];
   return vol;
 }
 
-vector<bool> Grid::getIsPeriodic() const {
+std::vector<bool> GridBase::getIsPeriodic() const {
   return pbc_;
 }
 
-vector<unsigned> Grid::getNbin() const {
+std::vector<unsigned> GridBase::getNbin() const {
   return nbin_;
 }
 
-vector<string> Grid::getArgNames() const {
+std::vector<std::string> GridBase::getArgNames() const {
   return argnames;
 }
 
-
-Grid::index_t Grid::getSize() const {
-  return maxsize_;
-}
-
-unsigned Grid::getDimension() const {
+unsigned GridBase::getDimension() const {
   return dimension_;
 }
 
 // we are flattening arrays using a column-major order
-Grid::index_t Grid::getIndex(const vector<unsigned> & indices) const {
+GridBase::index_t GridBase::getIndex(const std::vector<unsigned> & indices) const {
   plumed_dbg_assert(indices.size()==dimension_);
   for(unsigned int i=0; i<dimension_; i++)
     if(indices[i]>=nbin_[i]) {
@@ -193,14 +175,14 @@ Grid::index_t Grid::getIndex(const vector<unsigned> & indices) const {
   return index;
 }
 
-Grid::index_t Grid::getIndex(const vector<double> & x) const {
+GridBase::index_t GridBase::getIndex(const std::vector<double> & x) const {
   plumed_dbg_assert(x.size()==dimension_);
   return getIndex(getIndices(x));
 }
 
 // we are flattening arrays using a column-major order
-vector<unsigned> Grid::getIndices(index_t index) const {
-  vector<unsigned> indices(dimension_);
+std::vector<unsigned> GridBase::getIndices(index_t index) const {
+  std::vector<unsigned> indices(dimension_);
   index_t kk=index;
   indices[0]=(index%nbin_[0]);
   for(unsigned int i=1; i<dimension_-1; ++i) {
@@ -213,7 +195,7 @@ vector<unsigned> Grid::getIndices(index_t index) const {
   return indices;
 }
 
-void Grid::getIndices(index_t index, std::vector<unsigned>& indices) const {
+void GridBase::getIndices(index_t index, std::vector<unsigned>& indices) const {
   if (indices.size()!=dimension_) indices.resize(dimension_);
   index_t kk=index;
   indices[0]=(index%nbin_[0]);
@@ -226,48 +208,48 @@ void Grid::getIndices(index_t index, std::vector<unsigned>& indices) const {
   }
 }
 
-vector<unsigned> Grid::getIndices(const vector<double> & x) const {
+std::vector<unsigned> GridBase::getIndices(const std::vector<double> & x) const {
   plumed_dbg_assert(x.size()==dimension_);
-  vector<unsigned> indices(dimension_);
+  std::vector<unsigned> indices(dimension_);
   for(unsigned int i=0; i<dimension_; ++i) {
-    indices[i] = unsigned(floor((x[i]-min_[i])/dx_[i]));
+    indices[i] = unsigned(std::floor((x[i]-min_[i])/dx_[i]));
   }
   return indices;
 }
 
-void Grid::getIndices(const vector<double> & x, std::vector<unsigned>& indices) const {
+void GridBase::getIndices(const std::vector<double> & x, std::vector<unsigned>& indices) const {
   plumed_dbg_assert(x.size()==dimension_);
   if (indices.size()!=dimension_) indices.resize(dimension_);
   for(unsigned int i=0; i<dimension_; ++i) {
-    indices[i] = unsigned(floor((x[i]-min_[i])/dx_[i]));
+    indices[i] = unsigned(std::floor((x[i]-min_[i])/dx_[i]));
   }
 }
 
-vector<double> Grid::getPoint(const vector<unsigned> & indices) const {
+std::vector<double> GridBase::getPoint(const std::vector<unsigned> & indices) const {
   plumed_dbg_assert(indices.size()==dimension_);
-  vector<double> x(dimension_);
+  std::vector<double> x(dimension_);
   for(unsigned int i=0; i<dimension_; ++i) {
     x[i]=min_[i]+(double)(indices[i])*dx_[i];
   }
   return x;
 }
 
-vector<double> Grid::getPoint(index_t index) const {
+std::vector<double> GridBase::getPoint(index_t index) const {
   plumed_dbg_assert(index<maxsize_);
   return getPoint(getIndices(index));
 }
 
-vector<double> Grid::getPoint(const vector<double> & x) const {
+std::vector<double> GridBase::getPoint(const std::vector<double> & x) const {
   plumed_dbg_assert(x.size()==dimension_);
   return getPoint(getIndices(x));
 }
 
-void Grid::getPoint(index_t index,std::vector<double> & point) const {
+void GridBase::getPoint(index_t index,std::vector<double> & point) const {
   plumed_dbg_assert(index<maxsize_);
   getPoint(getIndices(index),point);
 }
 
-void Grid::getPoint(const std::vector<unsigned> & indices,std::vector<double> & point) const {
+void GridBase::getPoint(const std::vector<unsigned> & indices,std::vector<double> & point) const {
   plumed_dbg_assert(indices.size()==dimension_);
   plumed_dbg_assert(point.size()==dimension_);
   for(unsigned int i=0; i<dimension_; ++i) {
@@ -275,17 +257,16 @@ void Grid::getPoint(const std::vector<unsigned> & indices,std::vector<double> &
   }
 }
 
-void Grid::getPoint(const std::vector<double> & x,std::vector<double> & point) const {
+void GridBase::getPoint(const std::vector<double> & x,std::vector<double> & point) const {
   plumed_dbg_assert(x.size()==dimension_);
   getPoint(getIndices(x),point);
 }
 
-vector<Grid::index_t> Grid::getNeighbors
-(const vector<unsigned> &indices,const vector<unsigned> &nneigh)const {
+std::vector<GridBase::index_t> GridBase::getNeighbors(const std::vector<unsigned> &indices,const std::vector<unsigned> &nneigh)const {
   plumed_dbg_assert(indices.size()==dimension_ && nneigh.size()==dimension_);
 
-  vector<index_t> neighbors;
-  vector<unsigned> small_bin(dimension_);
+  std::vector<index_t> neighbors;
+  std::vector<unsigned> small_bin(dimension_);
 
   unsigned small_nbin=1;
   for(unsigned j=0; j<dimension_; ++j) {
@@ -293,8 +274,8 @@ vector<Grid::index_t> Grid::getNeighbors
     small_nbin*=small_bin[j];
   }
 
-  vector<unsigned> small_indices(dimension_);
-  vector<unsigned> tmp_indices;
+  std::vector<unsigned> small_indices(dimension_);
+  std::vector<unsigned> tmp_indices;
   for(unsigned index=0; index<small_nbin; ++index) {
     tmp_indices.resize(dimension_);
     unsigned kk=index;
@@ -322,24 +303,22 @@ vector<Grid::index_t> Grid::getNeighbors
   return neighbors;
 }
 
-vector<Grid::index_t> Grid::getNeighbors
-(const vector<double> & x,const vector<unsigned> & nneigh)const {
+std::vector<GridBase::index_t> GridBase::getNeighbors(const std::vector<double> & x,const std::vector<unsigned> & nneigh)const {
   plumed_dbg_assert(x.size()==dimension_ && nneigh.size()==dimension_);
   return getNeighbors(getIndices(x),nneigh);
 }
 
-vector<Grid::index_t> Grid::getNeighbors
-(index_t index,const vector<unsigned> & nneigh)const {
+std::vector<GridBase::index_t> GridBase::getNeighbors(index_t index,const std::vector<unsigned> & nneigh)const {
   plumed_dbg_assert(index<maxsize_ && nneigh.size()==dimension_);
   return getNeighbors(getIndices(index),nneigh);
 }
 
-void Grid::getSplineNeighbors(const vector<unsigned> & indices, vector<Grid::index_t>& neighbors, unsigned& nneighbors)const {
+void GridBase::getSplineNeighbors(const std::vector<unsigned> & indices, std::vector<GridBase::index_t>& neighbors, unsigned& nneighbors)const {
   plumed_dbg_assert(indices.size()==dimension_);
-  unsigned nneigh=unsigned(pow(2.0,int(dimension_)));
+  unsigned nneigh=unsigned(std::pow(2.0,int(dimension_)));
   if (neighbors.size()!=nneigh) neighbors.resize(nneigh);
 
-  vector<unsigned> nindices(dimension_);
+  std::vector<unsigned> nindices(dimension_);
   unsigned inind; nneighbors = 0;
   for(unsigned int i=0; i<nneigh; ++i) {
     unsigned tmp=i; inind=0;
@@ -354,12 +333,12 @@ void Grid::getSplineNeighbors(const vector<unsigned> & indices, vector<Grid::ind
   }
 }
 
-vector<Grid::index_t> Grid::getNearestNeighbors(const index_t index) const {
-  vector<index_t> nearest_neighs = vector<index_t>();
+std::vector<GridBase::index_t> GridBase::getNearestNeighbors(const index_t index) const {
+  std::vector<index_t> nearest_neighs = std::vector<index_t>();
   for (unsigned i = 0; i < dimension_; i++) {
-    vector<unsigned> neighsneeded = vector<unsigned>(dimension_, 0);
+    std::vector<unsigned> neighsneeded = std::vector<unsigned>(dimension_, 0);
     neighsneeded[i] = 1;
-    vector<index_t> singledim_nearest_neighs = getNeighbors(index, neighsneeded);
+    std::vector<index_t> singledim_nearest_neighs = getNeighbors(index, neighsneeded);
     for (unsigned j = 0; j < singledim_nearest_neighs.size(); j++) {
       index_t neigh = singledim_nearest_neighs[j];
       if (neigh != index) {
@@ -370,13 +349,12 @@ vector<Grid::index_t> Grid::getNearestNeighbors(const index_t index) const {
   return nearest_neighs;
 }
 
-vector<Grid::index_t> Grid::getNearestNeighbors(const vector<unsigned> &indices) const {
+std::vector<GridBase::index_t> GridBase::getNearestNeighbors(const std::vector<unsigned> &indices) const {
   plumed_dbg_assert(indices.size() == dimension_);
   return getNearestNeighbors(getIndex(indices));
 }
 
-
-void Grid::addKernel( const KernelFunctions& kernel ) {
+void GridBase::addKernel( const KernelFunctions& kernel ) {
   plumed_dbg_assert( kernel.ndim()==dimension_ );
   std::vector<unsigned> nneighb=kernel.getSupport( dx_ );
   std::vector<index_t> neighbors=getNeighbors( kernel.getCenter(), nneighb );
@@ -384,7 +362,7 @@ void Grid::addKernel( const KernelFunctions& kernel ) {
   std::vector<std::unique_ptr<Value>> vv( dimension_ );
   std::string str_min, str_max;
   for(unsigned i=0; i<dimension_; ++i) {
-    vv[i].reset(new Value());
+    vv[i]=Tools::make_unique<Value>();
     if( pbc_[i] ) {
       Tools::convert(min_[i],str_min);
       Tools::convert(max_[i],str_max);
@@ -411,57 +389,24 @@ void Grid::addKernel( const KernelFunctions& kernel ) {
   }
 }
 
-double Grid::getValue(index_t index) const {
-  plumed_dbg_assert(index<maxsize_);
-  return grid_[index];
-}
-
-double Grid::getMinValue() const {
-  double minval;
-  minval=DBL_MAX;
-  for(index_t i=0; i<grid_.size(); ++i) {
-    if(grid_[i]<minval)minval=grid_[i];
-  }
-  return minval;
-}
-
-double Grid::getMaxValue() const {
-  double maxval;
-  maxval=DBL_MIN;
-  for(index_t i=0; i<grid_.size(); ++i) {
-    if(grid_[i]>maxval)maxval=grid_[i];
-  }
-  return maxval;
-}
-
-
-double Grid::getValue(const vector<unsigned> & indices) const {
+double GridBase::getValue(const std::vector<unsigned> & indices) const {
   return getValue(getIndex(indices));
 }
 
-double Grid::getValue(const vector<double> & x) const {
+double GridBase::getValue(const std::vector<double> & x) const {
   if(!dospline_) {
     return getValue(getIndex(x));
   } else {
-    vector<double> der(dimension_);
+    std::vector<double> der(dimension_);
     return getValueAndDerivatives(x,der);
   }
 }
 
-double Grid::getValueAndDerivatives
-(index_t index, vector<double>& der) const {
-  plumed_dbg_assert(index<maxsize_ && usederiv_ && der.size()==dimension_);
-  der=der_[index];
-  return grid_[index];
-}
-
-double Grid::getValueAndDerivatives
-(const vector<unsigned> & indices, vector<double>& der) const {
+double GridBase::getValueAndDerivatives(const std::vector<unsigned> & indices, std::vector<double>& der) const {
   return getValueAndDerivatives(getIndex(indices),der);
 }
 
-double Grid::getValueAndDerivatives
-(const vector<double> & x, vector<double>& der) const {
+double GridBase::getValueAndDerivatives(const std::vector<double> & x, std::vector<double>& der) const {
   plumed_dbg_assert(der.size()==dimension_ && usederiv_);
 
   if(dospline_) {
@@ -472,14 +417,14 @@ double Grid::getValueAndDerivatives
     value=0.0;
     for(unsigned int i=0; i<dimension_; ++i) der[i]=0.0;
 
-    vector<unsigned> indices(dimension_);
+    std::vector<unsigned> indices(dimension_);
     getIndices(x, indices);
-    vector<double> xfloor(dimension_);
+    std::vector<double> xfloor(dimension_);
     getPoint(indices, xfloor);
-    vector<index_t> neigh; unsigned nneigh; getSplineNeighbors(indices, neigh, nneigh);
+    std::vector<index_t> neigh; unsigned nneigh; getSplineNeighbors(indices, neigh, nneigh);
 
 // loop over neighbors
-    vector<unsigned> nindices;
+    std::vector<unsigned> nindices;
     for(unsigned int ipoint=0; ipoint<nneigh; ++ipoint) {
       double grid=getValueAndDerivatives(neigh[ipoint],dder);
       getIndices(neigh[ipoint], nindices);
@@ -489,11 +434,11 @@ double Grid::getValueAndDerivatives
         int x0=1;
         if(nindices[j]==indices[j]) x0=0;
         double dx=getDx(j);
-        X=fabs((x[j]-xfloor[j])/dx-(double)x0);
+        X=std::abs((x[j]-xfloor[j])/dx-(double)x0);
         X2=X*X;
         X3=X2*X;
         double yy;
-        if(fabs(grid)<0.0000001) yy=0.0;
+        if(std::abs(grid)<0.0000001) yy=0.0;
         else yy=-dder[j]/grid;
         C[j]=(1.0-3.0*X2+2.0*X3) - (x0?-1.0:1.0)*yy*(X-2.0*X2+X3)*dx;
         D[j]=( -6.0*X +6.0*X2) - (x0?-1.0:1.0)*yy*(1.0-4.0*X +3.0*X2)*dx;
@@ -513,88 +458,23 @@ double Grid::getValueAndDerivatives
   }
 }
 
-void Grid::setValue(index_t index, double value) {
-  plumed_dbg_assert(index<maxsize_ && !usederiv_);
-  grid_[index]=value;
-}
-
-void Grid::setValue(const vector<unsigned> & indices, double value) {
+void GridBase::setValue(const std::vector<unsigned> & indices, double value) {
   setValue(getIndex(indices),value);
 }
 
-void Grid::setValueAndDerivatives
-(index_t index, double value, vector<double>& der) {
-  plumed_dbg_assert(index<maxsize_ && usederiv_ && der.size()==dimension_);
-  grid_[index]=value;
-  der_[index]=der;
-}
-
-void Grid::setValueAndDerivatives
-(const vector<unsigned> & indices, double value, vector<double>& der) {
+void GridBase::setValueAndDerivatives(const std::vector<unsigned> & indices, double value, std::vector<double>& der) {
   setValueAndDerivatives(getIndex(indices),value,der);
 }
 
-void Grid::addValue(index_t index, double value) {
-  plumed_dbg_assert(index<maxsize_ && !usederiv_);
-  grid_[index]+=value;
-}
-
-void Grid::addValue(const vector<unsigned> & indices, double value) {
+void GridBase::addValue(const std::vector<unsigned> & indices, double value) {
   addValue(getIndex(indices),value);
 }
 
-void Grid::addValueAndDerivatives
-(index_t index, double value, vector<double>& der) {
-  plumed_dbg_assert(index<maxsize_ && usederiv_ && der.size()==dimension_);
-  grid_[index]+=value;
-  for(unsigned int i=0; i<dimension_; ++i) der_[index][i]+=der[i];
-}
-
-void Grid::addValueAndDerivatives
-(const vector<unsigned> & indices, double value, vector<double>& der) {
+void GridBase::addValueAndDerivatives(const std::vector<unsigned> & indices, double value, std::vector<double>& der) {
   addValueAndDerivatives(getIndex(indices),value,der);
 }
 
-void Grid::scaleAllValuesAndDerivatives( const double& scalef ) {
-  if(usederiv_) {
-    for(index_t i=0; i<grid_.size(); ++i) {
-      grid_[i]*=scalef;
-      for(unsigned j=0; j<dimension_; ++j) der_[i][j]*=scalef;
-    }
-  } else {
-    for(index_t i=0; i<grid_.size(); ++i) grid_[i]*=scalef;
-  }
-}
-
-void Grid::logAllValuesAndDerivatives( const double& scalef ) {
-  if(usederiv_) {
-    for(index_t i=0; i<grid_.size(); ++i) {
-      grid_[i] = scalef*log(grid_[i]);
-      for(unsigned j=0; j<dimension_; ++j) der_[i][j] = scalef/der_[i][j];
-    }
-  } else {
-    for(index_t i=0; i<grid_.size(); ++i) grid_[i] = scalef*log(grid_[i]);
-  }
-}
-
-void Grid::setMinToZero() {
-  double min=grid_[0];
-  for(index_t i=1; i<grid_.size(); ++i) if(grid_[i]<min) min=grid_[i];
-  for(index_t i=0; i<grid_.size(); ++i) grid_[i] -= min;
-}
-
-void Grid::applyFunctionAllValuesAndDerivatives( double (*func)(double val), double (*funcder)(double valder) ) {
-  if(usederiv_) {
-    for(index_t i=0; i<grid_.size(); ++i) {
-      grid_[i]=func(grid_[i]);
-      for(unsigned j=0; j<dimension_; ++j) der_[i][j]=funcder(der_[i][j]);
-    }
-  } else {
-    for(index_t i=0; i<grid_.size(); ++i) grid_[i]=func(grid_[i]);
-  }
-}
-
-void Grid::writeHeader(OFile& ofile) {
+void GridBase::writeHeader(OFile& ofile) {
   for(unsigned i=0; i<dimension_; ++i) {
     ofile.addConstantField("min_" + argnames[i]);
     ofile.addConstantField("max_" + argnames[i]);
@@ -603,9 +483,14 @@ void Grid::writeHeader(OFile& ofile) {
   }
 }
 
+void Grid::clear() {
+  grid_.assign(maxsize_,0.0);
+  if(usederiv_) der_.assign(maxsize_*dimension_,0.0);
+}
+
 void Grid::writeToFile(OFile& ofile) {
-  vector<double> xx(dimension_);
-  vector<double> der(dimension_);
+  std::vector<double> xx(dimension_);
+  std::vector<double> der(dimension_);
   double f;
   writeHeader(ofile);
   for(index_t i=0; i<getSize(); ++i) {
@@ -627,7 +512,7 @@ void Grid::writeToFile(OFile& ofile) {
   }
 }
 
-void Grid::writeCubeFile(OFile& ofile, const double& lunit) {
+void GridBase::writeCubeFile(OFile& ofile, const double& lunit) {
   plumed_assert( dimension_==3 );
   ofile.printf("PLUMED CUBE FILE\n");
   ofile.printf("OUTER LOOP: X, MIDDLE LOOP: Y, INNER LOOP: Z\n");
@@ -649,10 +534,10 @@ void Grid::writeCubeFile(OFile& ofile, const double& lunit) {
   }
 }
 
-std::unique_ptr<Grid> Grid::create(const std::string& funcl, const std::vector<Value*> & args, IFile& ifile,
-                                   const vector<std::string> & gmin,const vector<std::string> & gmax,
-                                   const vector<unsigned> & nbin,bool dosparse, bool dospline, bool doder) {
-  std::unique_ptr<Grid> grid=Grid::create(funcl,args,ifile,dosparse,dospline,doder);
+std::unique_ptr<GridBase> GridBase::create(const std::string& funcl, const std::vector<Value*> & args, IFile& ifile,
+    const std::vector<std::string> & gmin,const std::vector<std::string> & gmax,
+    const std::vector<unsigned> & nbin,bool dosparse, bool dospline, bool doder) {
+  std::unique_ptr<GridBase> grid=GridBase::create(funcl,args,ifile,dosparse,dospline,doder);
   std::vector<unsigned> cbin( grid->getNbin() );
   std::vector<std::string> cmin( grid->getMin() ), cmax( grid->getMax() );
   for(unsigned i=0; i<args.size(); ++i) {
@@ -667,9 +552,9 @@ std::unique_ptr<Grid> Grid::create(const std::string& funcl, const std::vector<V
   return grid;
 }
 
-std::unique_ptr<Grid> Grid::create(const std::string& funcl, const std::vector<Value*> & args, IFile& ifile, bool dosparse, bool dospline, bool doder)
+std::unique_ptr<GridBase> GridBase::create(const std::string& funcl, const std::vector<Value*> & args, IFile& ifile, bool dosparse, bool dospline, bool doder)
 {
-  std::unique_ptr<Grid> grid;
+  std::unique_ptr<GridBase> grid;
   unsigned nvar=args.size(); bool hasder=false; std::string pstring;
   std::vector<int> gbin1(nvar); std::vector<unsigned> gbin(nvar);
   std::vector<std::string> labels(nvar),gmin(nvar),gmax(nvar);
@@ -703,11 +588,11 @@ std::unique_ptr<Grid> Grid::create(const std::string& funcl, const std::vector<V
     }
   }
 
-  if(!dosparse) {grid.reset(new Grid(funcl,args,gmin,gmax,gbin,dospline,doder));}
-  else {grid.reset(new SparseGrid(funcl,args,gmin,gmax,gbin,dospline,doder));}
+  if(!dosparse) {grid=Tools::make_unique<Grid>(funcl,args,gmin,gmax,gbin,dospline,doder);}
+  else {grid=Tools::make_unique<SparseGrid>(funcl,args,gmin,gmax,gbin,dospline,doder);}
 
-  vector<double> xx(nvar),dder(nvar);
-  vector<double> dx=grid->getDx();
+  std::vector<double> xx(nvar),dder(nvar);
+  std::vector<double> dx=grid->getDx();
   double f,x;
   while( ifile.scanField(funcl,f) ) {
     for(unsigned i=0; i<nvar; ++i) {
@@ -726,17 +611,61 @@ std::unique_ptr<Grid> Grid::create(const std::string& funcl, const std::vector<V
   return grid;
 }
 
-// Sparse version of grid with map
-void SparseGrid::clear() {
-  map_.clear();
+double Grid::getMinValue() const {
+  double minval;
+  minval=DBL_MAX;
+  for(index_t i=0; i<grid_.size(); ++i) {
+    if(grid_[i]<minval)minval=grid_[i];
+  }
+  return minval;
 }
 
-Grid::index_t SparseGrid::getSize() const {
-  return map_.size();
+double Grid::getMaxValue() const {
+  double maxval;
+  maxval=DBL_MIN;
+  for(index_t i=0; i<grid_.size(); ++i) {
+    if(grid_[i]>maxval)maxval=grid_[i];
+  }
+  return maxval;
 }
 
-Grid::index_t SparseGrid::getMaxSize() const {
-  return maxsize_;
+void Grid::scaleAllValuesAndDerivatives( const double& scalef ) {
+  if(usederiv_) {
+    for(index_t i=0; i<grid_.size(); ++i) {
+      grid_[i]*=scalef;
+      for(unsigned j=0; j<dimension_; ++j) der_[i*dimension_+j]*=scalef;
+    }
+  } else {
+    for(index_t i=0; i<grid_.size(); ++i) grid_[i]*=scalef;
+  }
+}
+
+void Grid::logAllValuesAndDerivatives( const double& scalef ) {
+  if(usederiv_) {
+    for(index_t i=0; i<grid_.size(); ++i) {
+      grid_[i] = scalef*std::log(grid_[i]);
+      for(unsigned j=0; j<dimension_; ++j) der_[i*dimension_+j] = scalef/der_[i*dimension_+j];
+    }
+  } else {
+    for(index_t i=0; i<grid_.size(); ++i) grid_[i] = scalef*std::log(grid_[i]);
+  }
+}
+
+void Grid::setMinToZero() {
+  double min=grid_[0];
+  for(index_t i=1; i<grid_.size(); ++i) if(grid_[i]<min) min=grid_[i];
+  for(index_t i=0; i<grid_.size(); ++i) grid_[i] -= min;
+}
+
+void Grid::applyFunctionAllValuesAndDerivatives( double (*func)(double val), double (*funcder)(double valder) ) {
+  if(usederiv_) {
+    for(index_t i=0; i<grid_.size(); ++i) {
+      grid_[i]=func(grid_[i]);
+      for(unsigned j=0; j<dimension_; ++j) der_[i*dimension_+j]=funcder(der_[i*dimension_+j]);
+    }
+  } else {
+    for(index_t i=0; i<grid_.size(); ++i) grid_[i]=func(grid_[i]);
+  }
 }
 
 double Grid::getDifferenceFromContour( const std::vector<double>& x, std::vector<double>& der ) const {
@@ -786,6 +715,54 @@ void Grid::findSetOfPointsOnContour(const double& target, const std::vector<bool
   }
 }
 
+/// OVERRIDES ARE BELOW
+
+Grid::index_t Grid::getSize() const {
+  return maxsize_;
+}
+
+double Grid::getValue(index_t index) const {
+  plumed_dbg_assert(index<maxsize_);
+  return grid_[index];
+}
+
+double Grid::getValueAndDerivatives(index_t index, std::vector<double>& der) const {
+  plumed_dbg_assert(index<maxsize_ && usederiv_ && der.size()==dimension_);
+  der.resize(dimension_);
+  for(unsigned i=0; i<dimension_; i++) der[i]=der_[dimension_*index+i];
+  return grid_[index];
+}
+
+void Grid::setValue(index_t index, double value) {
+  plumed_dbg_assert(index<maxsize_ && !usederiv_);
+  grid_[index]=value;
+}
+
+void Grid::setValueAndDerivatives(index_t index, double value, std::vector<double>& der) {
+  plumed_dbg_assert(index<maxsize_ && usederiv_ && der.size()==dimension_);
+  grid_[index]=value;
+  for(unsigned i=0; i<dimension_; i++) der_[dimension_*index+i]=der[i];
+}
+
+void Grid::addValue(index_t index, double value) {
+  plumed_dbg_assert(index<maxsize_ && !usederiv_);
+  grid_[index]+=value;
+}
+
+void Grid::addValueAndDerivatives(index_t index, double value, std::vector<double>& der) {
+  plumed_dbg_assert(index<maxsize_ && usederiv_ && der.size()==dimension_);
+  grid_[index]+=value;
+  for(unsigned int i=0; i<dimension_; ++i) der_[index*dimension_+i]+=der[i];
+}
+
+Grid::index_t SparseGrid::getSize() const {
+  return map_.size();
+}
+
+Grid::index_t SparseGrid::getMaxSize() const {
+  return maxsize_;
+}
+
 double SparseGrid::getValue(index_t index)const {
   plumed_assert(index<maxsize_);
   double value=0.0;
@@ -794,8 +771,7 @@ double SparseGrid::getValue(index_t index)const {
   return value;
 }
 
-double SparseGrid::getValueAndDerivatives
-(index_t index, vector<double>& der)const {
+double SparseGrid::getValueAndDerivatives(index_t index, std::vector<double>& der)const {
   plumed_assert(index<maxsize_ && usederiv_ && der.size()==dimension_);
   double value=0.0;
   for(unsigned int i=0; i<dimension_; ++i) der[i]=0.0;
@@ -811,8 +787,7 @@ void SparseGrid::setValue(index_t index, double value) {
   map_[index]=value;
 }
 
-void SparseGrid::setValueAndDerivatives
-(index_t index, double value, vector<double>& der) {
+void SparseGrid::setValueAndDerivatives(index_t index, double value, std::vector<double>& der) {
   plumed_assert(index<maxsize_ && usederiv_ && der.size()==dimension_);
   map_[index]=value;
   der_[index]=der;
@@ -823,8 +798,7 @@ void SparseGrid::addValue(index_t index, double value) {
   map_[index]+=value;
 }
 
-void SparseGrid::addValueAndDerivatives
-(index_t index, double value, vector<double>& der) {
+void SparseGrid::addValueAndDerivatives(index_t index, double value, std::vector<double>& der) {
   plumed_assert(index<maxsize_ && usederiv_ && der.size()==dimension_);
   map_[index]+=value;
   der_[index].resize(dimension_);
@@ -832,8 +806,8 @@ void SparseGrid::addValueAndDerivatives
 }
 
 void SparseGrid::writeToFile(OFile& ofile) {
-  vector<double> xx(dimension_);
-  vector<double> der(dimension_);
+  std::vector<double> xx(dimension_);
+  std::vector<double> der(dimension_);
   double f;
   writeHeader(ofile);
   ofile.fmtField(" "+fmt_);
@@ -857,6 +831,23 @@ void SparseGrid::writeToFile(OFile& ofile) {
   }
 }
 
+double SparseGrid::getMinValue() const {
+  double minval;
+  minval=0.0;
+  for(auto const & i : map_) {
+    if(i.second<minval) minval=i.second;
+  }
+  return minval;
+}
+
+double SparseGrid::getMaxValue() const {
+  double maxval;
+  maxval=0.0;
+  for(auto const & i : map_) {
+    if(i.second>maxval) maxval=i.second;
+  }
+  return maxval;
+}
 
 void Grid::projectOnLowDimension(double &val, std::vector<int> &vHigh, WeightBase * ptr2obj ) {
   unsigned i=0;
@@ -894,11 +885,11 @@ void Grid::projectOnLowDimension(double &val, std::vector<int> &vHigh, WeightBas
 
 Grid Grid::project(const std::vector<std::string> & proj, WeightBase *ptr2obj ) {
   // find extrema only for the projection
-  vector<string>   smallMin,smallMax;
-  vector<unsigned> smallBin;
-  vector<unsigned> dimMapping;
-  vector<bool> smallIsPeriodic;
-  vector<string> smallName;
+  std::vector<std::string>   smallMin,smallMax;
+  std::vector<unsigned> smallBin;
+  std::vector<unsigned> dimMapping;
+  std::vector<bool> smallIsPeriodic;
+  std::vector<std::string> smallName;
 
   // check if the two key methods are there
   WeightBase* pp = dynamic_cast<WeightBase*>(ptr2obj);
@@ -920,14 +911,14 @@ Grid Grid::project(const std::vector<std::string> & proj, WeightBase *ptr2obj )
       }
     }
   }
-  Grid smallgrid("projection",smallName,smallMin,smallMax,smallBin,false,false,true,smallIsPeriodic,smallMin,smallMax);
+  Grid smallgrid("projection",smallName,smallMin,smallMax,smallBin,false,false,smallIsPeriodic,smallMin,smallMax);
   // check that the two grids are commensurate
   for(unsigned i=0; i<dimMapping.size(); i++) {
     plumed_massert(  (smallgrid.getMax())[i] == (getMax())[dimMapping[i]],  "the two input grids are not compatible in max"   );
     plumed_massert(  (smallgrid.getMin())[i] == (getMin())[dimMapping[i]],  "the two input grids are not compatible in min"   );
     plumed_massert(  (smallgrid.getNbin())[i]== (getNbin())[dimMapping[i]], "the two input grids are not compatible in bin"   );
   }
-  vector<unsigned> toBeIntegrated;
+  std::vector<unsigned> toBeIntegrated;
   for(unsigned i=0; i<getArgNames().size(); i++) {
     bool doappend=true;
     for(unsigned j=0; j<dimMapping.size(); j++) {
@@ -935,12 +926,6 @@ Grid Grid::project(const std::vector<std::string> & proj, WeightBase *ptr2obj )
     }
     if(doappend)toBeIntegrated.push_back(i);
   }
-  //for(unsigned i=0;i<dimMapping.size();i++ ){
-  //     cerr<<"Dimension to preserve "<<dimMapping[i]<<endl;
-  //}
-  //for(unsigned i=0;i<toBeIntegrated.size();i++ ){
-  //     cerr<<"Dimension to integrate "<<toBeIntegrated[i]<<endl;
-  //}
 
   // loop over all the points in the Grid, find the corresponding fixed index, rotate over all the other ones
   for(unsigned i=0; i<smallgrid.getSize(); i++) {
@@ -951,19 +936,7 @@ Grid Grid::project(const std::vector<std::string> & proj, WeightBase *ptr2obj )
     // the vector vhigh now contains at the beginning the index of the low dimension and -1 for the values that need to be integrated
     double initval=0.;
     projectOnLowDimension(initval,vHigh, ptr2obj);
-    smallgrid.setValue(i,initval);
-  }
-  // reset to zero just for biasing (this option can be evtl enabled in a future...)
-  //double vmin;vmin=-smallgrid.getMinValue()+1;
-  for(unsigned i=0; i<smallgrid.getSize(); i++) {
-    //         //if(dynamic_cast<BiasWeight*>(ptr2obj)){
-    //         //        smallgrid.addValue(i,vmin);// go to 1
-    //         //}
-    double vv=smallgrid.getValue(i);
-    smallgrid.setValue(i,ptr2obj->projectOuterLoop(vv));
-    //         //if(dynamic_cast<BiasWeight*>(ptr2obj)){
-    //         //        smallgrid.addValue(i,-vmin);// bring back to the value
-    //         //}
+    smallgrid.setValue(i,ptr2obj->projectOuterLoop(initval));
   }
 
   return smallgrid;
@@ -1005,11 +978,11 @@ void Grid::mpiSumValuesAndDerivatives( Communicator& comm ) {
 }
 
 
-bool indexed_lt(pair<Grid::index_t, double> const &x, pair<Grid::index_t, double> const   &y) {
+bool indexed_lt(std::pair<Grid::index_t, double> const &x, std::pair<Grid::index_t, double> const   &y) {
   return x.second < y.second;
 }
 
-double Grid::findMaximalPathMinimum(const std::vector<double> &source, const std::vector<double> &sink) {
+double GridBase::findMaximalPathMinimum(const std::vector<double> &source, const std::vector<double> &sink) {
   plumed_dbg_assert(source.size() == dimension_);
   plumed_dbg_assert(sink.size() == dimension_);
   // Start and end indices
@@ -1097,15 +1070,15 @@ double Grid::findMaximalPathMinimum(const std::vector<double> &source, const std
   } else if (dimension_ > 1) {
     // Prepare calculation temporaries for Dijkstra's algorithm.
     // Minimal path costs from source to a given grid point
-    vector<double> mins_from_source = vector<double>(maxsize_, -1.0);
+    std::vector<double> mins_from_source = std::vector<double>(maxsize_, -1.0);
     // Heap for tracking available steps, steps are recorded as std::pairs of
     // an index and a value.
-    vector< pair<index_t, double> > next_steps;
-    pair<index_t, double> curr_indexed_val;
-    make_heap(next_steps.begin(), next_steps.end(), indexed_lt);
+    std::vector< std::pair<index_t, double> > next_steps;
+    std::pair<index_t, double> curr_indexed_val;
+    std::make_heap(next_steps.begin(), next_steps.end(), indexed_lt);
     // The search begins at the source index.
-    next_steps.push_back(pair<index_t, double>(source_idx, getValue(source_idx)));
-    push_heap(next_steps.begin(), next_steps.end(), indexed_lt);
+    next_steps.push_back(std::pair<index_t, double>(source_idx, getValue(source_idx)));
+    std::push_heap(next_steps.begin(), next_steps.end(), indexed_lt);
     // At first no points have been examined and the optimal path has not been found.
     index_t n_examined = 0;
     bool path_not_found = true;
@@ -1114,7 +1087,7 @@ double Grid::findMaximalPathMinimum(const std::vector<double> &source, const std
       // Examine the grid point currently most accessible from
       // the set of all previously explored grid points by popping
       // it from the top of the heap.
-      pop_heap(next_steps.begin(), next_steps.end(), indexed_lt);
+      std::pop_heap(next_steps.begin(), next_steps.end(), indexed_lt);
       curr_indexed_val = next_steps.back();
       next_steps.pop_back();
       n_examined++;
@@ -1132,7 +1105,7 @@ double Grid::findMaximalPathMinimum(const std::vector<double> &source, const std
       }
       // If the search is not over, add this grid point's neighbors to the
       // possible next points to search for the sink.
-      vector<index_t> neighs = getNearestNeighbors(curr_indexed_val.first);
+      std::vector<index_t> neighs = getNearestNeighbors(curr_indexed_val.first);
       for (unsigned k = 0; k < neighs.size(); k++) {
         index_t i = neighs[k];
         // If the neighbor has not already been added to the list of possible next steps,
@@ -1140,8 +1113,8 @@ double Grid::findMaximalPathMinimum(const std::vector<double> &source, const std
           // Set the cost to reach it via a path through the current point being examined.
           mins_from_source[i] = fmin(curr_indexed_val.second, getValue(i));
           // Add the neighboring point to the heap of potential next steps.
-          next_steps.push_back(pair<index_t, double>(i, mins_from_source[i]));
-          push_heap(next_steps.begin(), next_steps.end(), indexed_lt);
+          next_steps.push_back(std::pair<index_t, double>(i, mins_from_source[i]));
+          std::push_heap(next_steps.begin(), next_steps.end(), indexed_lt);
         }
       }
       // Move on to the next best looking step along any of the paths
